from paida.paida_core.PAbsorber import *
from paida.paida_core.IRangeSet import *
from paida.paida_core.PExceptions import *
import paida.paida_core.PTypes as PTypes

import types

class IFitData:
	def __init__(self):
		self._connection = []
		self._range = []
		self._binned = None
		self._dataDescription = ''
		
	def create1DConnection(self, data1, data2 = None, data3 = None):
		if (data1.__class__.__name__ == 'IHistogram1D') and (data2 == None) and (data3 == None):
			self._binned = True
			histogram = data1
			xAxis = histogram.axis()
			centers = []
			weights = []
			errors = []
			for xBinNumber in range(xAxis.bins()):
				weight = histogram.binHeight(xBinNumber)
				if weight == 0.0:
					xCenter = (xAxis.binLowerEdge(xBinNumber) + xAxis.binUpperEdge(xBinNumber)) / 2.0
				else:
					xCenter = histogram.binMean(xBinNumber)
				centers.append([xCenter])
				weights.append(weight)
				errors.append(histogram._binError2(xBinNumber))
			self._connection = [centers, weights, errors]
			self._range.append(IRangeSet())

		elif (data1.__class__.__name__ == 'ICloud1D') and (data2 == None) and (data3 == None):
			self._binned = False
			cloud = data1
			centers = []
			weights = []
			for entryNumber in range(cloud.entries()):
				xCenter = cloud.value(entryNumber)
				centers.append([xCenter])
				weights.append(cloud.weight(entryNumber))
			self._connection = [centers, weights]
			self._range.append(IRangeSet())

		elif (data1.__class__.__name__ == 'IProfile1D') and (data2 == None) and (data3 == None):
			self._binned = True
			profile = data1
			xAxis = profile.axis()
			centers = []
			weights = []
			errors = []
			for xBinNumber in range(xAxis.bins()):
				weight = profile.binHeight(xBinNumber)
				if weight == 0.0:
					xCenter = (xAxis.binLowerEdge(xBinNumber) + xAxis.binUpperEdge(xBinNumber)) / 2.0
				else:
					xCenter = profile.binMean(xBinNumber)
				centers.append([xCenter])
				weights.append(weight)
				errors.append(profile._binError2(xBinNumber))
			self._connection = [centers, weights, errors]
			self._range.append(IRangeSet())

		elif (data1.__class__.__name__ == 'IDataPointSet') and (type(data2) == types.IntType) and (type(data3) == types.IntType):
			self._binned = True
			dataPointSet = data1
			xIndex = data2
			valIndex = data3
			centers = []
			weights = []
			errors = []
			for offset in range(dataPointSet.size()):
				dataPoint = dataPointSet.point(offset)
				x = dataPoint.coordinate(xIndex)
				y = dataPoint.coordinate(valIndex)
				if y.errorMinus() == y.errorPlus():
					errors.append(y.errorMinus()**2)
				else:
					### Not good.
					raise IllegalArgumentException('This dataPoint (= dataPointSet[%s]) has asymmetric errors, and is not suitable for fitting.' % offset)
				centers.append([x.value()])
				weights.append(y.value())
			self._connection = [centers, weights, errors]
			self._range.append(IRangeSet())

		else:
			raise IllegalArgumentException('Invalid arguments.')
		
	def create2DConnection(self, data1, data2 = None, data3 = None, data4 = None):
		if (data1.__class__.__name__ == 'IHistogram2D') and (data2 == None) and (data3 == None) and (data4 == None):
			self._binned = True
			histogram = data1
			xAxis = histogram.xAxis()
			yAxis = histogram.yAxis()
			centers = []
			weights = []
			errors = []
			for xBinNumber in range(xAxis.bins()):
				for yBinNumber in range(yAxis.bins()):
					weight = histogram.binHeight(xBinNumber, yBinNumber)
					if weight == 0.0:
						xCenter = (xAxis.binLowerEdge(xBinNumber) + xAxis.binUpperEdge(xBinNumber)) / 2.0
						yCenter = (yAxis.binLowerEdge(yBinNumber) + yAxis.binUpperEdge(yBinNumber)) / 2.0
					else:
						xCenter = histogram.binMeanX(xBinNumber, yBinNumber)
						yCenter = histogram.binMeanY(xBinNumber, yBinNumber)
					centers.append([xCenter, yCenter])
					weights.append(weight)
					errors.append(histogram._binError2(xBinNumber, yBinNumber))
			self._connection = [centers, weights, errors]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())

		elif (data1.__class__.__name__ == 'IHistogram2D') and (type(data2) == types.IntType) and (type(data3) == types.IntType) and (data4 == None):
			self._binned = True
			histogram = data1
			xIndex = data2
			yIndex = data3
			xAxis = histogram.xAxis()
			yAxis = histogram.yAxis()
			centers = []
			weights = []
			errors = []
			for xBinNumber in range(xAxis.bins()):
				for yBinNumber in range(yAxis.bins()):
					weight = histogram.binHeight(xBinNumber, yBinNumber)
					if weight == 0.0:
						xCenter = (xAxis.binLowerEdge(xBinNumber) + xAxis.binUpperEdge(xBinNumber)) / 2.0
						yCenter = (yAxis.binLowerEdge(yBinNumber) + yAxis.binUpperEdge(yBinNumber)) / 2.0
					else:
						xCenter = histogram.binMeanX(xBinNumber, yBinNumber)
						yCenter = histogram.binMeanY(xBinNumber, yBinNumber)
					if (xIndex == 0) and (yIndex == 1):
						centers.append([xCenter, yCenter])
					elif (xIndex == 1) and (yIndex == 0):
						centers.append([yCenter, xCenter])
					else:
						raise IllegalArgumentException('Invalid index pair (xIndex, yIndex) = (%s, %s).' % (xIndex, yIndex))
					weights.append(weight)
					errors.append(histogram._binError2(xBinNumber, yBinNumber))
			self._connection = [centers, weights, errors]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
		
		elif (data1.__class__.__name__ == 'ICloud2D') and (data2 == None) and (data3 == None) and (data4 == None):
			self._binned = False
			cloud = data1
			centers = []
			weights = []
			for entryNumber in range(cloud.entries()):
				xCenter = cloud.valueX(entryNumber)
				yCenter = cloud.valueY(entryNumber)
				centers.append([xCenter, yCenter])
				weights.append(cloud.weight(entryNumber))
			self._connection = [centers, weights]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())

		elif (data1.__class__.__name__ == 'ICloud2D') and (type(data2) == types.IntType) and (type(data3) == types.IntType) and (data4 == None):
			self._binned = False
			cloud = data1
			xIndex = data2
			yIndex = data3
			centers = []
			weights = []
			for entryNumber in range(cloud.entries()):
				xCenter = cloud.valueX(entryNumber)
				yCenter = cloud.valueY(entryNumber)
				if (xIndex == 0) and (yIndex == 1):
					centers.append([xCenter, yCenter])
				elif (xIndex == 1) and (yIndex == 0):
					centers.append([yCenter, xCenter])
				else:
					raise IllegalArgumentException('Invalid index pair (xIndex, yIndex) = (%s, %s).' % (xIndex, yIndex))
				weights.append(cloud.weight(entryNumber))
			self._connection = [centers, weights]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
		
		elif (data1.__class__.__name__ == 'IProfile2D') and (data2 == None) and (data3 == None) and (data4 == None):
			self._binned = True
			profile = data1
			xAxis = profile.xAxis()
			yAxis = profile.yAxis()
			centers = []
			weights = []
			errors = []
			for xBinNumber in range(xAxis.bins()):
				for yBinNumber in range(yAxis.bins()):
					weight = profile.binHeight(xBinNumber, yBinNumber)
					if weight == 0.0:
						xCenter = (xAxis.binLowerEdge(xBinNumber) + xAxis.binUpperEdge(xBinNumber)) / 2.0
						yCenter = (yAxis.binLowerEdge(yBinNumber) + yAxis.binUpperEdge(yBinNumber)) / 2.0
					else:
						xCenter = profile.binMeanX(xBinNumber, yBinNumber)
						yCenter = profile.binMeanY(xBinNumber, yBinNumber)
					centers.append([xCenter, yCenter])
					weights.append(weight)
					errors.append(profile._binError2(xBinNumber, yBinNumber))
			self._connection = [centers, weights, errors]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())

		elif (data1.__class__.__name__ == 'IProfile2D') and (type(data2) == types.IntType) and (type(data3) == types.IntType) and (data4 == None):
			self._binned = True
			profile = data1
			xIndex = data2
			yIndex = data3
			xAxis = profile.xAxis()
			yAxis = profile.yAxis()
			centers = []
			weights = []
			errors = []
			for xBinNumber in range(xAxis.bins()):
				for yBinNumber in range(yAxis.bins()):
					weight = profile.binHeight(xBinNumber, yBinNumber)
					if weight == 0.0:
						xCenter = (xAxis.binLowerEdge(xBinNumber) + xAxis.binUpperEdge(xBinNumber)) / 2.0
						yCenter = (yAxis.binLowerEdge(yBinNumber) + yAxis.binUpperEdge(yBinNumber)) / 2.0
					else:
						xCenter = profile.binMeanX(xBinNumber, yBinNumber)
						yCenter = profile.binMeanY(xBinNumber, yBinNumber)
					if (xIndex == 0) and (yIndex == 1):
						centers.append([xCenter, yCenter])
					elif (xIndex == 1) and (yIndex == 0):
						centers.append([yCenter, xCenter])
					else:
						raise IllegalArgumentException('Invalid index pair (xIndex, yIndex) = (%s, %s).' % (xIndex, yIndex))
					weights.append(weight)
					errors.append(profile._binError2(xBinNumber, yBinNumber))
			self._connection = [centers, weights, errors]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())

		elif (data1.__class__.__name__ == 'IDataPointSet') and (type(data2) == types.IntType) and (type(data3) == types.IntType) and (type(data4) == types.IntType):
			self._binned = True
			dataPointSet = data1
			xIndex = data2
			yIndex = data3
			valIndex = data4
			centers = []
			weights = []
			errors = []
			for offset in range(dataPointSet.size()):
				dataPoint = dataPointSet.point(offset)
				x = dataPoint.coordinate(xIndex)
				y = dataPoint.coordinate(yIndex)
				z = dataPoint.coordinate(valIndex)
				if z.errorMinus() == z.errorPlus():
					errors.append(z.errorMinus()**2)
				else:
					### Not good.
					raise IllegalArgumentException('This dataPoint (= dataPointSet[%s]) has asymmetric errors, and is not suitable for fitting.' % offset)
				centers.append([x.value(), y.value()])
				weights.append(z.value())
			self._connection = [centers, weights, errors]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
			
		else:
			raise IllegalArgumentException('Invalid arguments.')

	def create3DConnection(self, data1, data2 = None, data3 = None, data4 = None, data5 = None):
		if (data1.__class__.__name__ == 'IHistogram3D') and (data2 == None) and (data3 == None) and (data4 == None) and (data5 == None):
			self._binned = True
			histogram = data1
			xAxis = histogram.xAxis()
			yAxis = histogram.yAxis()
			zAxis = histogram.zAxis()
			centers = []
			weights = []
			errors = []
			for xBinNumber in range(xAxis.bins()):
				for yBinNumber in range(yAxis.bins()):
					for zBinNumber in range(zAxis.bins()):
						weight = histogram.binHeight(xBinNumber, yBinNumber, zBinNumber)
						if weight == 0.0:
							xCenter = (xAxis.binLowerEdge(xBinNumber) + xAxis.binUpperEdge(xBinNumber)) / 2.0
							yCenter = (yAxis.binLowerEdge(yBinNumber) + yAxis.binUpperEdge(yBinNumber)) / 2.0
							zCenter = (zAxis.binLowerEdge(zBinNumber) + zAxis.binUpperEdge(zBinNumber)) / 2.0
						else:
							xCenter = histogram.binMeanX(xBinNumber, yBinNumber, zBinNumber)
							yCenter = histogram.binMeanY(xBinNumber, yBinNumber, zBinNumber)
							zCenter = histogram.binMeanZ(xBinNumber, yBinNumber, zBinNumber)
						centers.append([xCenter, yCenter, zCenter])
						weights.append(weight)
						errors.append(histogram._binError2(xBinNumber, yBinNumber, zBinNumber))
			self._connection = [centers, weights, errors]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
		
		elif (data1.__class__.__name__ == 'IHistogram3D') and (type(data2) == types.IntType) and (type(data3) == types.IntType) and (type(data4) == types.IntType) and (data5 == None):
			self._binned = True
			histogram = data1
			xIndex = data2
			yIndex = data3
			zIndex = data4
			xAxis = histogram.xAxis()
			yAxis = histogram.yAxis()
			zAxis = histogram.zAxis()
			centers = []
			weights = []
			errors = []
			for xBinNumber in range(xAxis.bins()):
				for yBinNumber in range(yAxis.bins()):
					for zBinNumber in range(zAxis.bins()):
						weight = histogram.binHeight(xBinNumber, yBinNumber, zBinNumber)
						if weight == 0.0:
							xCenter = (xAxis.binLowerEdge(xBinNumber) + xAxis.binUpperEdge(xBinNumber)) / 2.0
							yCenter = (yAxis.binLowerEdge(yBinNumber) + yAxis.binUpperEdge(yBinNumber)) / 2.0
							zCenter = (zAxis.binLowerEdge(zBinNumber) + zAxis.binUpperEdge(zBinNumber)) / 2.0
						else:
							xCenter = histogram.binMeanX(xBinNumber, yBinNumber, zBinNumber)
							yCenter = histogram.binMeanY(xBinNumber, yBinNumber, zBinNumber)
							zCenter = histogram.binMeanZ(xBinNumber, yBinNumber, zBinNumber)
						if (xIndex == 0) and (yIndex == 1) and (zIndex == 2):
							centers.append([xCenter, yCenter, zCenter])
						elif (xIndex == 0) and (yIndex == 2) and (zIndex == 1):
							centers.append([xCenter, zCenter, yCenter])
						elif (xIndex == 1) and (yIndex == 0) and (zIndex == 2):
							centers.append([yCenter, xCenter, zCenter])
						elif (xIndex == 1) and (yIndex == 2) and (zIndex == 0):
							centers.append([zCenter, xCenter, yCenter])
						elif (xIndex == 2) and (yIndex == 0) and (zIndex == 1):
							centers.append([yCenter, zCenter, xCenter])
						elif (xIndex == 2) and (yIndex == 1) and (zIndex == 0):
							centers.append([zCenter, yCenter, xCenter])
						else:
							raise IllegalArgumentException('Invalid index pair (xIndex, yIndex, zIndex) = (%s, %s, %s).' % (xIndex, yIndex, zIndex))
						weights.append(weight)
						errors.append(histogram._binError2(xBinNumber, yBinNumber, zBinNumber))
			self._connection = [centers, weights, errors]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
		
		elif (data1.__class__.__name__ == 'ICloud3D') and (data2 == None) and (data3 == None) and (data4 == None) and (data5 == None):
			self._binned = False
			cloud = data1
			centers = []
			weights = []
			for entryNumber in range(cloud.entries()):
				xCenter = cloud.valueX(entryNumber)
				yCenter = cloud.valueY(entryNumber)
				zCenter = cloud.valueZ(entryNumber)
				centers.append([xCenter, yCenter, zCenter])
				weights.append(cloud.weight(entryNumber))
			self._connection = [centers, weights]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())

		elif (data1.__class__.__name__ == 'ICloud3D') and (type(data2) == types.IntType) and (type(data3) == types.IntType) and (type(data4) == types.IntType) and (data5 == None):
			self._binned = False
			cloud = data1
			xIndex = data2
			yIndex = data3
			zIndex = data4
			centers = []
			weights = []
			for entryNumber in range(cloud.entries()):
				xCenter = cloud.valueX(entryNumber)
				yCenter = cloud.valueY(entryNumber)
				zCenter = cloud.valueZ(entryNumber)
				if (xIndex == 0) and (yIndex == 1) and (zIndex == 2):
					centers.append([xCenter, yCenter, zCenter])
				elif (xIndex == 0) and (yIndex == 2) and (zIndex == 1):
					centers.append([xCenter, zCenter, yCenter])
				elif (xIndex == 1) and (yIndex == 0) and (zIndex == 2):
					centers.append([yCenter, xCenter, zCenter])
				elif (xIndex == 1) and (yIndex == 2) and (zIndex == 0):
					centers.append([zCenter, xCenter, yCenter])
				elif (xIndex == 2) and (yIndex == 0) and (zIndex == 1):
					centers.append([yCenter, zCenter, xCenter])
				elif (xIndex == 2) and (yIndex == 1) and (zIndex == 0):
					centers.append([zCenter, yCenter, xCenter])
				else:
					raise IllegalArgumentException('Invalid index pair (xIndex, yIndex, zIndex) = (%s, %s, %s).' % (xIndex, yIndex, zIndex))
				weights.append(cloud.weight(entryNumber))
			self._connection = [centers, weights]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
			
		elif (data1.__class__.__name__ == 'IDataPointSet') and (type(data2) == types.IntType) and (type(data3) == types.IntType) and (type(data4) == types.IntType) and (type(data5) == types.IntType):
			self._binned = True
			dataPointSet = data1
			xIndex = data2
			yIndex = data3
			zIndex = data4
			valIndex = data5
			centers = []
			weights = []
			errors = []
			for offset in range(dataPointSet.size()):
				dataPoint = dataPointSet.point(offset)
				x = dataPoint.coordinate(xIndex)
				y = dataPoint.coordinate(yIndex)
				z = dataPoint.coordinate(zIndex)
				val = dataPoint.coordinate(valIndex)
				if val.errorMinus() == val.errorPlus():
					errors.append(val.errorMinus())
				else:
					### Not good.
					raise IllegalArgumentException('This dataPoint (= dataPointSet[%s]) has asymmetric errors, and is not suitable for fitting.' % offset)
				centers.append([x.value(), y.value(), z.value()])
				weights.append(val.value())
			self._connection = [centers, weights, errors]
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
			self._range.append(IRangeSet())
		
		else:
			raise IllegalArgumentException('Invalid arguments.')

	def createConnection(self, data1 = None, data2 = None, data3 = None):
		if (data1.__class__.__name__ == 'ITuple') and (type(data2) == types.ListType) and (data3 == None):
			self._binned = False
			iTuple = data1
			colData = data2
			centers = []
			weights = []
			length = iTuple.rows()
			if length == -1:
				raise IllegalArgumentException('This ITuple has no row.')
			iTuple.start()
			for offset in range(length):
				if iTuple.next():
					center = []
					if type(colData[0]) in types.StringTypes:
						for columnName in colData:
							column = iTuple.findColumn(columnName)
							columnType = iTuple.columnType(column)
							if columnType == PTypes.Double:
								center.append(iTuple.getDouble(column))
							elif columnType == PTypes.Float:
								center.append(iTuple.getFloat(column))
							elif columnType == PTypes.Int:
								center.append(iTuple.getInt(column))
							elif columnType == PTypes.Short:
								center.append(iTuple.getShort(column))
							elif columnType == PTypes.Long:
								center.append(iTuple.getLong(column))
							elif columnType == PTypes.Char:
								center.append(iTuple.getChar(column))
							elif columnType == PTypes.Byte:
								center.append(iTuple.getByte(column))
							elif columnType == PTypes.Boolean:
								center.append(iTuple.getBoolean(column))
							elif columnType == PTypes.String:
								center.append(iTuple.getString(column))
							elif columnType == PTypes.Object:
								center.append(iTuple.getObject(column))
							elif columnType == PTypes.Tuple:
								center.append(iTuple.getTuple(column))
							else:
								raise IllegalArgumentException('Illegal column type %s' % columnType)
					elif colData[0].__class__.__name__ == 'IEvaluator':
						for evaluator in colData:
							center.append(evaluator.evaluateDouble())
					else:
						raise IllegalArgumentException('Illegal list data type.')
					centers.append(center[:-1])
					weights.append(center[-1])
				else:
					raise IllegalArgumentException('This ITuple structure is broken.')
			self._connection = [centers, weights]
			for i in range(len(colNames) - 1):
				self._range.append(IRangeSet())

		elif (data1.__class__.__name__ == 'IDataPointSet') and (type(data2) == types.ListType) and (type(data3) == types.IntType):
			self._binned = True
			dataPointSet = data1
			indices = data2
			valIndex = data3
			centers = []
			weights = []
			errors = []
			for offset in range(dataPointSet.size()):
				dataPoint = dataPointSet.point(offset)
				val = dataPoint.coordinate(valIndex)
				if val.errorMinus() == val.errorPlus():
					errors.append(val.errorMinus()**2)
				else:
					### Not good.
					raise IllegalArgumentException('This dataPoint (= dataPointSet[%s]) has asymmetric errors, and is not suitable for fitting.' % offset)
				center = []
				if indices == []:
					center.append(offset)
				else:
					for index in indices:
						center.append(dataPoint.coordinate(index).value())
				centers.append(center)
				weights.append(val.value())
			self._connection = [centers, weights, errors]
			for i in range(len(indices)):
				self._range.append(IRangeSet())
		
		else:
			raise IllegalArgumentException('Invalid arguments.')

	def reset(self):
		self._connection = []
		self._range = []
		self._binned = None
		self._dataDescription = ''
		
	def dimension(self):
		return len(self._range)
	
	def dataDescription(self):
		return self._dataDescription

	def range(self, index):
		try:
			return self._range[index]
		except IndexError:
			raise IllegalArgumentException('The Range does not exist.')


